"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("@zilliqa-js/util");
var util_2 = require("./util");
var bech32_1 = require("./bech32");
/**
 * @enum AddressType
 */
var AddressType;
(function (AddressType) {
    AddressType["bytes20"] = "bytes20";
    AddressType["bytes20Hex"] = "bytes20Hex";
    AddressType["checkSum"] = "checkSum";
    AddressType["base58"] = "base58";
    AddressType["bech32"] = "bech32";
    AddressType["bech32TestNet"] = "bech32TestNet";
})(AddressType || (AddressType = {}));
exports.AddressType = AddressType;
var ZilAddress = /** @class */ (function () {
    function ZilAddress(raw) {
        this.raw = raw;
        this.bytes20 = '0x';
        this.getAddressType();
    }
    ZilAddress.isValidBech32 = function (address) {
        return new ZilAddress(address).isBech32();
    };
    ZilAddress.isValidBech32TestNet = function (address) {
        return new ZilAddress(address).isBech32TestNet();
    };
    ZilAddress.isValidBase58 = function (address) {
        return new ZilAddress(address).isBase58();
    };
    ZilAddress.isValidChecksum = function (address) {
        return new ZilAddress(address).isChecksum();
    };
    ZilAddress.isValidByte20 = function (address) {
        return new ZilAddress(address).isBytes20();
    };
    ZilAddress.isValidByte20Hex = function (address) {
        return new ZilAddress(address).isBytes20Hex();
    };
    Object.defineProperty(ZilAddress.prototype, "bytes20Hex", {
        get: function () {
            return "0x" + this.bytes20;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ZilAddress.prototype, "checkSum", {
        get: function () {
            return util_2.toChecksumAddress(this.bytes20);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ZilAddress.prototype, "bech32", {
        get: function () {
            return bech32_1.toBech32Address(this.bytes20);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ZilAddress.prototype, "bech32TestNet", {
        get: function () {
            return bech32_1.toBech32Address(this.bytes20, 'tzil');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ZilAddress.prototype, "base58", {
        get: function () {
            return util_2.encodeBase58(this.bytes20);
        },
        enumerable: true,
        configurable: true
    });
    ZilAddress.prototype.isBytes20 = function () {
        return this.addressType === AddressType.bytes20;
    };
    ZilAddress.prototype.isBytes20Hex = function () {
        return this.addressType === AddressType.bytes20Hex;
    };
    ZilAddress.prototype.isChecksum = function () {
        return this.addressType === AddressType.checkSum;
    };
    ZilAddress.prototype.isBase58 = function () {
        return this.addressType === AddressType.base58;
    };
    ZilAddress.prototype.isBech32 = function () {
        return this.addressType === AddressType.bech32;
    };
    ZilAddress.prototype.isBech32TestNet = function () {
        return this.addressType === AddressType.bech32TestNet;
    };
    ZilAddress.prototype.getAddressType = function () {
        var addrBool = util_1.validation.isAddress(this.raw);
        var base58Bool = util_1.validation.isBase58(this.raw);
        var bech32Bool = util_1.validation.isBech32(this.raw);
        var bech32TestNetBool = util_1.validation.isBech32TestNet(this.raw);
        var checksumBool = util_2.isValidChecksumAddress(this.raw);
        if (addrBool === true && checksumBool === false) {
            this.addressType = AddressType.bytes20;
            this.bytes20 = this.raw.startsWith('0x')
                ? this.raw.substring(2)
                : this.raw;
            return;
        }
        if (addrBool === true && checksumBool === true) {
            this.addressType = AddressType.checkSum;
            this.bytes20 = this.raw.toLowerCase().substring(2);
            return;
        }
        if (bech32TestNetBool === true &&
            util_1.validation.isAddress(bech32_1.fromBech32Address(this.raw, 'tzil'))) {
            this.addressType = AddressType.bech32;
            var decoded = bech32_1.fromBech32Address(this.raw, 'tzil').toLowerCase();
            this.bytes20 = decoded.startsWith('0x') ? decoded.substring(2) : decoded;
            return;
        }
        if (bech32Bool === true &&
            util_1.validation.isAddress(bech32_1.fromBech32Address(this.raw))) {
            this.addressType = AddressType.bech32;
            var decoded = bech32_1.fromBech32Address(this.raw).toLowerCase();
            this.bytes20 = decoded.startsWith('0x') ? decoded.substring(2) : decoded;
            return;
        }
        if (base58Bool === true && util_1.validation.isAddress(util_2.decodeBase58(this.raw))) {
            this.addressType = AddressType.base58;
            var decoded = util_2.decodeBase58(this.raw).toLowerCase();
            this.bytes20 = decoded.startsWith('0x') ? decoded.substring(2) : decoded;
            return;
        }
        throw new Error('unknown address');
    };
    return ZilAddress;
}());
exports.ZilAddress = ZilAddress;
//# sourceMappingURL=zilAddress.js.map